@@ -1,4 +1,4 @@
-#' @title Risk set on an equidistant distant grid
+#' @title Risk Set on an Equidistant Distant Grid
 #' @description Get the risk set at each bin over an equidistant distant grid.
 #'
 #' @param Y vector of outcome variable returned from \code{\link{Surv}}.
@@ -11,14 +11,12 @@
 #'
 #' @return
 #' a list with the following elements
-#' \describe{
-#' \item{\code{risk_sets}}{list of lists with one for each bin. Each of the sub lists have indices that corresponds to the entries of \code{Y} that are at risk in the bin.}
-#' \item{\code{min_start}}{start time of the first bin.}
-#' \item{\code{I_len}}{length of each bin.}
-#' \item{\code{d}}{number of bins.}
-#' \item{\code{is_event_in}}{indices for which bin an observation \code{Y} is an event. \code{-1} if the individual does not die in any of the bins.}
-#' \item{\code{is_for_discrete_model}}{value of \code{is_for_discrete_model} argument.}
-#' }
+#' \item{risk_sets}{list of lists with one for each bin. Each of the sub lists have indices that corresponds to the entries of \code{Y} that are at risk in the bin.}
+#' \item{min_start}{start time of the first bin.}
+#' \item{I_len}{length of each bin.}
+#' \item{d}{number of bins.}
+#' \item{is_event_in}{indices for which bin an observation \code{Y} is an event. \code{-1} if the individual does not die in any of the bins.}
+#' \item{is_for_discrete_model}{value of \code{is_for_discrete_model} argument.}
 #'
 #' @examples
 #'# small toy example with time-varying covariates
@@ -60,11 +58,9 @@ get_risk_obj = function(
   # set exactly to boundaries where values are very close and difference is
   # likely due to floating point operations
   start_order = order(start, method = "radix") - 1L
-  start <-
-    round_if_almost_eq(start, start_order, c(min_start, event_times_in))
+  start <- round_if_almost_eq(start, start_order, c(min_start, event_times_in))
   stop_order = order(stop, method = "radix") - 1L
-  stop <-
-    round_if_almost_eq(stop, stop_order, c(min_start, event_times_in))
+  stop <- round_if_almost_eq(stop, stop_order, c(min_start, event_times_in))
 
   if(n_threads == 1 || min_chunk > n_ids){
     return(
