@@ -1,10 +1,8 @@
 #ifndef RESAMPLERS
 #define RESAMPLERS
 
-#include "PF_data.h"
-#include "particles.h"
-#include "../sample_funcs.h"
 #include "PF_utils.h"
+#include "../sample_funcs.h"
 #include "../utils.h"
 
 
@@ -59,23 +57,27 @@ protected:
  Each "resampler" has a static function which:
     1) computes log re-sampling weights assuming that weights are computed
     2) samples according to re-sampling weights
-    3) returns re-sampled indices if effective sample size is low. Otherwise
-       return an index for each element in the cloud. A boolean argument is
-       used to indicate if sampling is made
+    3) returns re-sampled indices if effective sample size is low through the
+       `outcome` argument. The actual returned object contains intermediate
+       object to avoid recomputation in e.g., the importance sampler. If the
+       effective sample size i large than the `outcome` argument is  an index
+       for each element in the cloud. The boolean argument is used to indicate
+       if sampling is performed.
 */
 
 /*
- Non-auxiliary particle filter where we just to the weights as they are. I.e.
- set:
-  beta_j = w_{j - 1}
+ Non-auxiliary particle filter where we just to the weights as they are. E.g.
+ set
+    beta_j = w_{j,t - 1}
+ in the forward particle filter.
 */
 
 template<typename densities, bool is_forward>
 class None_AUX_resampler : private resampler_base<systematic_resampling> {
 public:
   inline static nothing resampler(
-      const PF_data &data, cloud &PF_cloud, unsigned int t, arma::uvec &outcome,
-      bool &did_resample){
+      densities &dens_cal, const PF_data &data, cloud &PF_cloud,
+      unsigned int t, arma::uvec &outcome, bool &did_resample){
     /* Compute effective sample size (ESS) */
     arma::vec weights(PF_cloud.size());
     double ESS = 0;
@@ -97,15 +99,16 @@ public:
 
 /*
   Auxiliary particle filter with weights as in the end of page 462 of:
-    Fearnhead, P., Wyncoll, D., & Tawn, J. (2010). A sequential smoothing algorithm with linear computational cost. Biometrika, 97(2), 447-464.
+    Fearnhead, P., Wyncoll, D., & Tawn, J. (2010). A sequential smoothing
+    algorithm with linear computational cost. Biometrika, 97(2), 447-464.
 */
 
 template<typename densities, bool is_forward>
 class AUX_resampler_normal_approx_w_cloud_mean : private resampler_base<systematic_resampling> {
 public:
   inline static input_for_normal_apprx_w_cloud_mean resampler(
-      const PF_data &data, cloud &PF_cloud, unsigned int t, arma::uvec &outcome,
-      bool &did_resample){
+      densities &dens_cal, const PF_data &data, cloud &PF_cloud,
+      unsigned int t, arma::uvec &outcome, bool &did_resample){
     /* Find weighted mean estimate */
     arma::vec alpha_bar = PF_cloud.get_weigthed_mean();
 
@@ -113,65 +116,68 @@ public:
     auto &Q = data.Q_proposal;
     auto ans = compute_mu_n_Sigma_from_normal_apprx_w_cloud_mean
       <densities, is_forward>
-      (data, t, Q, alpha_bar, PF_cloud);
+      (dens_cal, data, t, Q, alpha_bar, PF_cloud);
 
-    const arma::mat *Q_numerator_chol_inv;
-    arma::vec *a_0_scaled;
-    double bw_w1, bw_w2;
-    if(is_forward){
-      Q_numerator_chol_inv = &data.Q.chol_inv;
-
-      // avoid wmaybe-uninitialized
-      bw_w1 = bw_w2 = std::numeric_limits<double>::quiet_NaN();
-
-    } else {
-      bw_w1 = (double)(t + 1) / (t + 2);
-      bw_w2 =              1. / (t + 2);
-      arma::vec tmp_vec = data.a_0 * bw_w2;
-
-      a_0_scaled = new arma::vec(std::move(tmp_vec));
-      arma::mat tmp = data.Q.chol_inv / sqrt((double)(t + 1) / (t + 2));
-      Q_numerator_chol_inv = new arma::mat(std::move(tmp));
-
-    }
-
-    /* Compute sampling weights*/
+    /* Compute sampling weights */
     double max_weight =  -std::numeric_limits<double>::max();
-    auto it_cl = PF_cloud.begin();
-    auto it_mu_j = ans.mu_js.begin();
     unsigned int n_elem = PF_cloud.size();
-    for(unsigned int i = 0; i != n_elem; ++i, ++it_cl, ++it_mu_j){
-      double log_prob_y_given_state = densities::log_prob_y_given_state(
-        data, *it_mu_j, t);
+    arma::uvec r_set = get_risk_set(data, t);
+#ifdef _OPENMP
+#pragma omp parallel
+{
+#endif
+    double my_max_weight = -std::numeric_limits<double>::max();
+
+#ifdef _OPENMP
+#pragma omp for schedule(static)
+#endif
+    for(unsigned int i = 0; i < n_elem; ++i){ // loop over cloud elements
+      auto it_cl = PF_cloud.begin() + i;
+      auto it_mu_j = ans.mu_js.begin() + i;
+
+      double log_prob_y_given_state =
+        densities::log_prob_y_given_state(
+          data, data.err_state->map(*it_mu_j).sv, t, r_set, false);
       double log_prop_transition;
       if(is_forward){
         log_prop_transition = dmvnrm_log(
-          *it_mu_j, it_cl->get_state(), *Q_numerator_chol_inv);
+          *it_mu_j,
+          data.err_state_inv->map(it_cl->get_state()).sv,
+          data.Q.chol_inv);
 
       } else {
-        arma::vec mean = bw_w1 * it_cl->get_state() + *a_0_scaled;
+        arma::vec mean = data.bw_mean(t, it_cl->get_state());
         log_prop_transition = dmvnrm_log(
-          *it_mu_j, mean, *Q_numerator_chol_inv);
+          *it_mu_j, data.err_state_inv->map(mean).sv,
+          data.bw_covar(t).chol_inv);
 
       }
 
       double log_prop_proposal = dmvnrm_log(
-        *it_mu_j, *it_mu_j /* Notice same */, ans.sigma_chol_inv /* Notice Sigma*/);
+        *it_mu_j, *it_mu_j /* Notice same */,
+        ans.sigma_chol_inv /* Notice Sigma*/);
 
       it_cl->log_resampling_weight =
         it_cl->log_weight + log_prop_transition + log_prob_y_given_state
         - log_prop_proposal;
 
-      max_weight = MAX(it_cl->log_resampling_weight, max_weight);
-    }
+      my_max_weight = MAX(it_cl->log_resampling_weight, my_max_weight);
+    } // end loop over cloud elements
 
-    if(!is_forward){
-      delete Q_numerator_chol_inv;
-      delete a_0_scaled;
+#ifdef _OPENMP
+#pragma omp critical(aux_resampler_normal_approx_w_cloud_mean_lock)
+{
+#endif
+    max_weight = MAX(my_max_weight, max_weight);
+#ifdef _OPENMP
+}
+} // end omp parallel
+#endif
 
-    }
-
-    auto norm_out = normalize_log_resampling_weight<true, true>(PF_cloud, max_weight);
+    auto norm_out =
+      normalize_log_resampling_weight
+      <true, true>
+      (PF_cloud, max_weight);
     outcome = sample(data, norm_out.weights, norm_out.ESS, did_resample);
 
     return ans;
@@ -181,41 +187,21 @@ public:
 /*
   Auxiliary particle filter with weights as in the end of page 462 of the
   following paper with Taylor expansion around the parent particle:
-    Fearnhead, P., Wyncoll, D., & Tawn, J. (2010). A sequential smoothing algorithm with linear computational cost. Biometrika, 97(2), 447-464.
+    Fearnhead, P., Wyncoll, D., & Tawn, J. (2010). A sequential smoothing
+    algorithm with linear computational cost. Biometrika, 97(2), 447-464.
 */
 
 template<typename densities, bool is_forward>
 class AUX_resampler_normal_approx_w_particles : private resampler_base<systematic_resampling> {
 public:
   inline static input_for_normal_apprx_w_particle_mean resampler(
-      const PF_data &data, cloud &PF_cloud, unsigned int t, arma::uvec &outcome,
-      bool &did_resample){
+      densities &dens_cal, const PF_data &data, cloud &PF_cloud,
+      unsigned int t, arma::uvec &outcome, bool &did_resample){
     /* compute means and covariances */
     auto &Q = data.Q_proposal;
     auto ans = compute_mu_n_Sigma_from_normal_apprx_w_particles
       <densities, is_forward>
-      (data, t, Q, PF_cloud);
-
-    arma::vec *a_0_scaled;
-    double bw_w1, bw_w2;
-    const arma::mat *Q_numerator_chol_inv;
-    if(is_forward){
-      Q_numerator_chol_inv = &data.Q.chol_inv;
-
-      // avoid wmaybe-uninitialized
-      bw_w1 = bw_w2 = std::numeric_limits<double>::quiet_NaN();
-      a_0_scaled = nullptr;
-
-    } else {
-      bw_w1 = (double)(t + 1) / (t + 2);
-      bw_w2 =              1. / (t + 2);
-      arma::vec tmp_vec = data.a_0 * bw_w2;
-
-      a_0_scaled = new arma::vec(std::move(tmp_vec));
-      arma::mat tmp = data.Q.chol_inv / sqrt((double)(t + 1) / (t + 2));
-      Q_numerator_chol_inv = new arma::mat(std::move(tmp));
-
-    }
+      (dens_cal, data, t, Q, PF_cloud);
 
     /* Compute sampling weights */
     double max_weight =  -std::numeric_limits<double>::max();
@@ -223,48 +209,57 @@ public:
     unsigned int n_elem = PF_cloud.size();
     arma::uvec r_set = get_risk_set(data, t);
 #ifdef _OPENMP
-#pragma omp parallel for schedule(static)
+#pragma omp parallel
+{
 #endif
-    for(unsigned int i = 0; i < n_elem; ++i){
+    double my_max_weight = -std::numeric_limits<double>::max();
+
+#ifdef _OPENMP
+#pragma omp for schedule(static)
+#endif
+    for(unsigned int i = 0; i < n_elem; ++i){ // loop over cloud elements
       auto it_cl = &PF_cloud[i];
       auto it_ans = &ans[i];
 
       double log_prob_y_given_state = densities::log_prob_y_given_state(
-        data, it_ans->mu, t, r_set, false);
+        data, data.err_state->map(it_ans->mu).sv, t, r_set, false);
 
       double log_prop_transition;
       if(is_forward){
         log_prop_transition = dmvnrm_log(
-          it_ans->mu, it_cl->get_state(), *Q_numerator_chol_inv);
+          it_ans->mu,
+          // will failed if there are fixed effects in the state vector
+          data.err_state_inv->map(it_cl->get_state()).sv,
+          data.Q.chol_inv);
 
       } else {
-        arma::vec mean = bw_w1 * it_cl->get_state() + *a_0_scaled;
+        arma::vec mean = data.bw_mean(t, it_cl->get_state());
         log_prop_transition = dmvnrm_log(
-          it_ans->mu, mean, *Q_numerator_chol_inv);
+          it_ans->mu, data.err_state_inv->map(mean).sv,
+          data.bw_covar(t).chol_inv);
 
       }
 
       double log_prop_proposal = dmvnrm_log(
-        it_ans->mu, it_ans->mu /* Notice same */, it_ans->sigma_chol_inv /* Notice Sigma*/);
+        it_ans->mu, it_ans->mu /* Notice same */,
+        it_ans->sigma_chol_inv /* Notice Sigma*/);
 
       it_cl->log_resampling_weight =
       it_cl->log_weight + log_prop_transition + log_prob_y_given_state
         - log_prop_proposal;
+
+      my_max_weight = MAX(it_cl->log_resampling_weight, my_max_weight);
+    } // end loop over cloud elements
+
 #ifdef _OPENMP
-#pragma omp critical(resampler_aux_norm_particle_lock)
+#pragma omp critical(aux_resampler_normal_approx_w_particles_lock)
 {
 #endif
-      max_weight = MAX(it_cl->log_resampling_weight, max_weight);
+    max_weight = MAX(my_max_weight, max_weight);
 #ifdef _OPENMP
 }
+} // end omp parallel
 #endif
-    }
-
-    if(!is_forward){
-      delete Q_numerator_chol_inv;
-      delete a_0_scaled;
-
-    }
 
     auto norm_out = normalize_log_resampling_weight<true, true>(PF_cloud, max_weight);
     outcome = sample(data, norm_out.weights, norm_out.ESS, did_resample);
