@@ -45,7 +45,7 @@ public:
       U = arma::mat(org.covar_dim, org.covar_dim);
 
       z_dot = arma::mat(
-        org.space_dim, // TODO: can be changed to dimension of covariates
+        org.state_dim, // TODO: can be changed to dimension of covariates
         n_in_last_set, arma::fill::zeros);
       H_diag_inv = arma::vec(n_in_last_set);
     }
@@ -115,9 +115,7 @@ inline void EKF_filter_worker<T>::operator()(){
   for(arma::uvec::const_iterator it = first; it != last; it++, i++){
     const arma::vec x_(org.X.colptr(*it), org.covar_dim, false);
     const double w = org.weights(*it);
-    const double offset =
-      (org.any_fixed_in_M_step) ?
-      arma::dot(org.fixed_parems, org.fixed_terms.col(*it)) : 0.;
+    const double offset = org.fixed_effects(*it);
     const double eta = arma::dot(dynamic_coefs, x_) + offset;
     const bool do_die = org.is_event_in_bin(*it) == bin_number;
     const double at_risk_length =
@@ -139,7 +137,7 @@ inline void EKF_filter_worker<T>::operator()(){
     if(compute_z_and_H){
       dat.H_diag_inv(i) = 1 / var;
       arma::vec tmp(x_ * mu_eta);
-      dat.z_dot.col(i) = org.lp_map_inv(tmp).sv;
+      dat.z_dot.col(i) = org.state_lp_inv->map(tmp).sv;
     }
   }
 
@@ -176,11 +174,11 @@ void EKF_solver<T>::solve(){
     bin_tstop += delta_t;
     // E-step: Prediction step
     org.a_t_less_s.col(t - 1) =
-      org.state_trans_map(org.a_t_t_s.col(t - 1)).sv;
+      org.state_trans->map(org.a_t_t_s.col(t - 1)).sv;
 
     org.V_t_less_s.slice(t - 1) =
-      org.state_trans_map(org.V_t_t_s.slice(t - 1)).sv +
-      delta_t * org.err_state_map(org.Q).sv;
+      org.state_trans->map(org.V_t_t_s.slice(t - 1)).sv +
+      delta_t * org.err_state->map(org.Q).sv;
 
     if(org.debug){
       std::stringstream str;
@@ -206,7 +204,7 @@ void EKF_solver<T>::solve(){
     while(true){
       ++n_NR_it;
 
-      arma::vec dynamic_coef(org.lp_map(i_a_t).sv);
+      arma::vec dynamic_coef(org.state_lp->map(i_a_t).sv);
       parallel_filter_step(
         r_set.begin(), r_set.end(), dynamic_coef,
         t == org.d, t - 1, bin_tstart, bin_tstop);
@@ -226,13 +224,13 @@ void EKF_solver<T>::solve(){
       }
 
       // E-step: scoring step: update values
-      auto U = org.lp_map_inv(p_dat->U);
+      auto U = org.state_lp_inv->map(p_dat->U);
       inv_sympd(
         org.V_t_t_s.slice(t) , V_t_less_s_inv + U.sv,
         org.use_pinv,
         "ddhazard_fit_cpp estimation error: Failed to compute inverse for V_(t|t)");
 
-      auto u = org.lp_map_inv(p_dat->u);
+      auto u = org.state_lp_inv->map(p_dat->u);
       org.a_t_t_s.col(t) =
         org.V_t_t_s.slice(t) * (
             U.sv * i_a_t + update_term + (org.LR * u.sv));
@@ -263,7 +261,7 @@ void EKF_solver<T>::solve(){
     }
 
     org.B_s.slice(t - 1) =
-      org.state_trans_map(org.V_t_t_s.slice(t - 1), right).sv * V_t_less_s_inv;
+      org.state_trans->map(org.V_t_t_s.slice(t - 1), right).sv * V_t_less_s_inv;
 
     if(org.debug){
       std::stringstream str;
@@ -277,28 +275,28 @@ void EKF_solver<T>::solve(){
     }
 
     if(t == org.d){
-      auto U = org.lp_map_inv(p_dat->U);
+      auto U = org.state_lp_inv->map(p_dat->U);
       arma::mat tmp_inv_mat;
       inv(tmp_inv_mat,
-          arma::eye<arma::mat>(org.space_dim, org.space_dim) +
+          arma::eye<arma::mat>(org.state_dim, org.state_dim) +
             U.sv * org.V_t_less_s.slice(t - 1),
           org.use_pinv, "ddhazard_fit_cpp estimation error: Failed to invert intermediate for K_d matrix");
 
       p_dat->K_d = org.V_t_less_s.slice(t - 1) * (
         tmp_inv_mat * p_dat->z_dot * diagmat(p_dat->H_diag_inv));
       // Parenthesis is key here to avoid making a n x n matrix for large n
-      p_dat->K_d = (org.state_trans_map(
+      p_dat->K_d = (org.state_trans->map(
         org.V_t_less_s.slice(t - 1), left).sv *
         p_dat->z_dot  * diagmat(p_dat->H_diag_inv) *
         p_dat->z_dot.t()) * p_dat->K_d;
       p_dat->K_d =
-        org.state_trans_map(org.V_t_less_s.slice(t - 1), left).sv *
+        org.state_trans->map(org.V_t_less_s.slice(t - 1), left).sv *
         p_dat->z_dot  * diagmat(p_dat->H_diag_inv) -  p_dat->K_d;
 
       org.lag_one_cov.slice(t - 1) =
-        (arma::eye<arma::mat>(org.space_dim, org.space_dim) -
+        (arma::eye<arma::mat>(org.state_dim, org.state_dim) -
         p_dat->K_d * p_dat->z_dot.t()) *
-        org.state_trans_map(org.V_t_t_s.slice(t - 1), left).sv;
+        org.state_trans->map(org.V_t_t_s.slice(t - 1), left).sv;
     }
   }
 }
