@@ -1,55 +1,86 @@
 # Run before test
 # See https://github.com/hadley/testthat/issues/544#issuecomment-260053774
 
+# function to test that expression does not throw an error
+expect_no_error = function(expr, env = parent.frame()){
+  call <- match.call()
+  eval(bquote(expect_error(.(call[[2]]), NA)), envir = env)
+}
+
+# expect_no_error(1 / "a")
+# expect_no_error(1 / 1)
+
+# load data example Fahrmeier (1994)
+get_head_neck_cancer_data <- function(){
+  is_censored = c(6, 27, 34, 36, 42, 46, 48:51,
+                  51 + c(15, 30:28, 33, 35:37, 39, 40, 42:45))
+  head_neck_cancer = data.frame(
+    id = 1:96,
+    start = rep(0, 96),
+    stop = c(
+      1, 2, 2, rep(3, 6), 4, 4, rep(5, 8),
+      rep(6, 7), 7, 8, 8, 8,
+      9, 9, 10, 10, 10, 11, 14, 14, 14, 15, 18, 18, 20, 20, 37,
+      37, 38, 41, 45, 47, 47,
+
+      2, 2, 3, rep(4, 4), rep(5, 5), rep(6, 5),
+      7, 7, 7, 9, 10, 11, 12, 15, 16, 18, 18, 18, 21,
+      21, 24, 25, 27, 36, 41, 44, 52, 54, 59, 59, 63, 67, 71, 76),
+    event = !(1:96 %in% is_censored),
+    group = factor(c(rep(1, 45 + 6), rep(2, 45))))
+
+  head_neck_cancer$group = factor(head_neck_cancer$group, levels = c(2, 1))
+
+  head_neck_cancer
+}
+
+########
+# PBC data set from survival with the time-variying covariates
+# See: https://cran.r-project.org/web/packages/survival/vignettes/timedep.pdf
+
+get_pbc2_data <- function(){
+  pbc <- survival::pbc
+  pbcseq <- survival::pbcseq
+
+  # avoid notes with CRAN tests
+  id <- NULL
+  sex <- NULL
+  time <- NULL
+  status <- NULL
+  edema <- NULL
+  age <- NULL
+  event <- NULL
+  tdc <- NULL
+  day <- NULL
+  albumin <- NULL
+  protime <- NULL
+  bili <- NULL
+
+  temp <- subset(pbc, id <= 312, select=c(id, sex, time, status, edema, age))
+  pbc2 <- survival::tmerge(
+    temp, temp, id=id, death = event(time, status))
+  pbc2 <- survival::tmerge(
+    pbc2, pbcseq, id=id, albumin = tdc(day, albumin),
+    protime = tdc(day, protime), bili = tdc(day, bili))
+  pbc2 <- pbc2[, c("id", "tstart", "tstop", "death", "sex", "edema",
+                   "age", "albumin", "protime", "bili")]
+
+  pbc2
+}
+
 options(useFancyQuotes = FALSE)
 
 if(interactive()){
   library(dynamichazard); library(testthat)
 
-  if(grepl("testthat$", getwd()))
-    source("../../R/test_utils.R") else source("R/test_utils.R")
-
-  exp_model_names <- with(environment(ddhazard), exp_model_names)
-
-  # test_LAPACK_BLAS_wrapper.R
-  chol_rank_one_update_test <- with(environment(ddhazard), chol_rank_one_update_test)
-  square_tri_inv_test <- with(environment(ddhazard), square_tri_inv_test)
-  symmetric_mat_chol_test <- with(environment(ddhazard), symmetric_mat_chol_test)
-  tri_mat_times_vec_test <- with(environment(ddhazard), tri_mat_times_vec_test)
-  sym_mat_rank_one_update_test <- with(environment(ddhazard), sym_mat_rank_one_update_test)
-  solve_w_precomputed_chol_test <- with(environment(ddhazard), solve_w_precomputed_chol_test)
-
-  # test_SMA.R
-  SMA_hepler_logit_compute_length <-
-    with(environment(ddhazard), SMA_hepler_logit_compute_length)
-  SMA_hepler_exp_compute_length <-
-    with(environment(ddhazard), SMA_hepler_exp_compute_length)
-
-  # testbigglm_wrapper.R
-  bigglm_updateQR_rcpp <- with(environment(ddhazard), bigglm_updateQR_rcpp)
-  bigglm_regcf_rcpp <- with(environment(ddhazard), bigglm_regcf_rcpp)
-
-  # testboot_est.R
-  get_frac_n_weights <- with(environment(ddhazard), get_frac_n_weights)
-
-  # testdesign_mat_and_risk_obj.R
-  get_permu_data_exp <-  with(environment(ddhazard), get_permu_data_exp)
-  get_permu_data_rev_exp <- with(environment(ddhazard), get_permu_data_rev_exp)
-  get_order_data_exp <-  with(environment(ddhazard), get_order_data_exp)
-  get_order_data_rev_exp <- with(environment(ddhazard), get_order_data_rev_exp)
-  get_design_matrix <- environment(ddhazard)$get_design_matrix
-
-  # test_cpp_utils.R
-  lambert_W0_test <- environment(ddhazard)$lambert_W0_test
-  trunc_eta_exponential_test <- environment(ddhazard)$trunc_eta_exponential_test
-  trunc_eta_exponential_test_log_eps <- environment(ddhazard)$trunc_eta_exponential_test_log_eps
+  list2env(as.list(asNamespace("dynamichazard")), envir = environment())
 }
 
 # wrapper for expect_know_...
 qu <- quote({
   file <- local({
     dir <- if(!interactive()) "./previous_results/" else
-      paste0(stringr::str_match(getwd(), ".+dynamichazard"),
+      paste0(gsub("(.+dynamichazard)(.*)", "\\1", getwd()),
              "/tests/testthat/previous_results/")
 
     paste0(dir, file)
@@ -93,8 +124,9 @@ save_to_test <- function(obj, file_name, tolerance = sqrt(.Machine$double.eps)){
     print(head(sort(unlist(lapply(obj, object.size)), decreasing = T))) else
       print(object.size(obj))
 
+  root <- gsub("(.+dynamichazard)(.*)", "\\1", getwd())
   out_file <- paste0(
-    stringr::str_match(getwd(), ".+dynamichazard"), "/tests/testthat/previous_results/", file_name, ".RDS")
+    root, "/tests/testthat/previous_results/", file_name, ".RDS")
   saveRDS(obj, compress = T, out_file)
 
   cat("RDS file size is ", file.size(out_file) / 1000, "KB\n", sep = "")
@@ -124,8 +156,9 @@ read_to_test <- function(file_name){
 }
 
 read_to_test_get_file_w_path <- function(file_name){
+  root <- gsub("(.+dynamichazard)(.*)", "\\1", getwd())
   path <- if(!interactive()) "./previous_results/" else
-    paste0(stringr::str_match(getwd(), ".+dynamichazard"), "/tests/testthat/previous_results/")
+    paste0(root, "/tests/testthat/previous_results/")
 
   paste0(path, file_name, ".RDS")
 }
